package test.driver;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.apache.log4j.BasicConfigurator;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import uk.co.keang.callerid.driver.CTICallerID;
import uk.co.keang.callerid.driver.CallSimulator;
import uk.co.keang.callerid.driver.CallerIDEvent;
import uk.co.keang.callerid.driver.ICallerIDListener;

public class CTICallerIDTest
{
protected int callReceived;
private Exception excThrown;

@BeforeClass
public static void setUpBeforeClass() throws Exception
    {
    BasicConfigurator.configure();
    }

@Before
public void setUp() throws Exception
    {
    excThrown = null;
    callReceived = 0;
    }

@Test
public void testAddCallerIDListener2() throws IOException
    {
    CallSimulator sim = new CallSimulator();
    CTICallerID callID = new CTICallerID(sim);

    callID.addCallerIDListener(new ICallerIDListener()
        {
        @Override
        public void incomingCall(CallerIDEvent event)
            {
            callReceived++;
            System.out.println("  The call is: "+event);
            }

        @Override
        public void callerIDException(Exception e)
            {
            excThrown = e;
            }
        });

    int numCalls = 3;

    for ( int i = 0; i < numCalls; i++ )
        {
        sim.callNextNumber();
        }

    sleep(1200);

    sim.close();
    callID.close();

    assertNull(excThrown);
    assertTrue(callReceived == numCalls);
    }

@Test(expected=IllegalArgumentException.class)
public void testBadCheckSum() throws Exception
    {
    CallSimulator sim = new CallSimulator();
    CTICallerID callID = new CTICallerID(sim);

    callID.addCallerIDListener(new ICallerIDListener()
        {
        @Override
        public void incomingCall(CallerIDEvent event)
            {
            throw new AssertionError("Shouldn't recieve an incoming call event");
            }

        @Override
        public void callerIDException(Exception e)
            {
            excThrown = e;
            }
        });

    sim.sendBadChecksum();

    sleep(500);

    sim.close();
    callID.close();

    if ( excThrown != null )
        throw excThrown;

    assertTrue("Shouldn't get here", false);
    }

@Test(expected=IllegalArgumentException.class)
public void testTruncated() throws Exception
    {
    CallSimulator sim = new CallSimulator();
    CTICallerID callID = new CTICallerID(sim);

    callID.addCallerIDListener(new ICallerIDListener()
        {
        @Override
        public void incomingCall(CallerIDEvent event)
            {
            throw new AssertionError("Shouldn't recieve an incoming call event");
            }

        @Override
        public void callerIDException(Exception e)
            {
            excThrown = e;
            }
        });

    sim.sendTruncated();

    sleep(500);

    sim.close();
    callID.close();

    if ( excThrown != null )
        throw excThrown;

    assertTrue("Shouldn't get here", false);
    }

/**
 * @param i
 *
 */
private void sleep(long time)
    {
    try
        {
        Thread.sleep(time);
        }
    catch ( InterruptedException exc )
        {
        // do nothing
        }
    }

@Test
public void testInterruptedCalls()
    {
    CallSimulator sim = new CallSimulator();
    CTICallerID callID = new CTICallerID(sim);

    callID.addCallerIDListener(new ICallerIDListener()
        {
        @Override
        public void incomingCall(CallerIDEvent event)
            {
            callReceived++;
            System.out.println("  The call is: "+event);
            }

        @Override
        public void callerIDException(Exception e)
            {
            excThrown = e;
            }
        });

    // send good tel num
    int numCalls = 1;
    sim.callNextNumber();
    sleep(500);
    assertNull(excThrown);
    assertTrue(callReceived == numCalls);

    // send bad checksum
    sim.sendBadChecksum();
    sleep(500);
    assertNotNull(excThrown);
    assertTrue(callReceived == numCalls);

    // send good tel num
    excThrown = null;
    numCalls++;
    sim.callNextNumber();
    sleep(500);
    assertNull(excThrown);
    assertTrue(callReceived == numCalls);

    // send truncated num
    sim.sendTruncated();
    sleep(500);
    assertNotNull(excThrown);
    assertTrue(callReceived == numCalls);

    // send good tel num
    excThrown = null;
    numCalls++;
    sim.callNextNumber();
    sleep(500);
    assertNull(excThrown);
    assertTrue(callReceived == numCalls);

    sim.close();
    callID.close();
    }

@Test
public void testNumWitheld()
    {
    CallSimulator sim = new CallSimulator();
    CTICallerID callID = new CTICallerID(sim);

    callID.addCallerIDListener(new ICallerIDListener()
        {
        @Override
        public void incomingCall(CallerIDEvent event)
            {
            callReceived++;
            System.out.println("  The call is: "+event);
            }

        @Override
        public void callerIDException(Exception e)
            {
            excThrown = e;
            }
        });

    // send good tel num
    int numCalls = 1;
    sim.callNextNumber();
    sleep(500);
    assertNull(excThrown);
    assertTrue(callReceived == numCalls);

    // send num withHeld
    excThrown = null;
    numCalls++;
    sim.callNumberWithHeld();
    sleep(500);
    assertNull(excThrown);
    assertTrue(callReceived == numCalls);

    // send good tel num
    excThrown = null;
    numCalls++;
    sim.callNextNumber();
    sleep(500);
    assertNull(excThrown);
    assertTrue(callReceived == numCalls);

    sim.close();
    callID.close();
    }
}
